/*
 * Created on 16.9.2006. - last revision: 2.4.2007.
 * (C) 2007. Milan Milanovic, GOOD OLD AI Laboratory, Faculty of Organizational Sciences, University of Belgrade.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  (See the file "COPYING" that is included with this source distribution.)
 */
package prjaoadlwithcm.atl;

import java.io.File;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.internal.resources.Workspace;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginRegistry;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.ui.dialogs.WorkspaceResourceDialog;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.m2m.atl.drivers.uml24atl.AtlUML2ModelHandler;
import org.eclipse.m2m.atl.drivers.emf4atl.AtlEMFModelHandler;
import org.eclipse.m2m.atl.drivers.emf4atl.ASMEMFModel;
import org.eclipse.m2m.atl.drivers.emf4atl.EMFModelLoader;
import org.eclipse.m2m.atl.engine.vm.AtlLauncher;
import org.eclipse.m2m.atl.engine.vm.AtlModelHandler;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;

import org.eclipse.ui.ide.undo.CreateProjectOperation;
import org.eclipse.ui.ide.undo.WorkspaceUndoUtil;
import org.eclipse.ui.internal.wizards.newresource.ResourceMessages;
import org.eclipse.uml2.uml.resource.UMLResource;

import org.eclipse.gmf.runtime.emf.core.internal.resources.PathmapManager;


import prjaoadlwithcm.atl.ATLTransformationError;


/**
 * Main class for running AO-ADL to Theme/UML or to plain UML transformations 
 * programatically, based on Milan Milanovic (milan [at] milanovic.org) 
 * ATLTransformations Template Bundle class
 * 
 * @author Luis Fernandez
 * @version 1.0
 */
public class ATLTransformations {
	private static ATLTransformations atlTransformation = null;
	
	 
	//private final static String resourcePath = "/org.eclipse.jwt.transformations.bpmn/src/org/eclipse/jwt/transformations/bpmn/resources/";
 
	private EMFModelLoader modelLoader;
	private AtlEMFModelHandler modelHandler;
	private URI aoadl_ModelResource;
	private String uml_ModelResource;
	private String strAOADL_PROFILE;
	private String strTHEMEUML_PROFILE;

	private URL aoadl2themeuml_TransfoResource;
	private URL uml2aoadl_TransfoResource;
	private URL LIBAOADL2ThemeUML;
	private URL LIBUMLClone;
	
	private ASMEMFModel AOADLMetamodel;
	private ASMEMFModel UMLMetamodel;
	private ASMEMFModel AOADL_PROFILE;
	private ASMEMFModel THEMEUML_PROFILE;

	
	public static ATLTransformations getATLTransformations(){
		if(atlTransformation == null){
			return new ATLTransformations();
		}
		
		return atlTransformation;
	}

	private void createAOADLResources() {
		modelHandler = (AtlEMFModelHandler) AtlModelHandler.getDefault(AtlModelHandler.AMH_EMF);
		modelLoader = (EMFModelLoader)modelHandler.createModelLoader();
		
		aoadl_ModelResource = URI.createPlatformPluginURI("prjAOADLwithCM/src/prjaoadlwithcm/atl/model/AOADL.ecore", false);
		uml_ModelResource = "uri:http://www.eclipse.org/uml2/3.0.0/UML";		
 		
		uml2aoadl_TransfoResource = ATLTransformations.class.getResource("transformations/patterns2CMaoadl.asm");

	}
	
	private void createThemeUMLResources(String path) {
		modelHandler = (AtlEMFModelHandler) AtlModelHandler.getDefault(AtlModelHandler.AMH_EMF);
		modelLoader = (EMFModelLoader)modelHandler.createModelLoader();
		
		aoadl_ModelResource = URI.createPlatformPluginURI("prjAOADLwithCM/src/prjaoadlwithcm/atl/model/AOADL.ecore", false);
		uml_ModelResource = "uri:http://www.eclipse.org/uml2/2.1.0/UML";		
 
		strAOADL_PROFILE = path + "AOADL.profile.uml";
		strTHEMEUML_PROFILE = path + "ThemeUML.profile.uml";
		
		aoadl2themeuml_TransfoResource = ATLTransformations.class.getResource("transformations/aoadlThemeUMLplusAspects.asm");

		LIBAOADL2ThemeUML = ATLTransformations.class.getResource("lib/aoadl2themeumlLIB.asm");
		LIBUMLClone = ATLTransformations.class.getResource("lib/umlCloneLIB.asm");
	}
	
	private void initAOADLMetamodels(Map<String, Object> models) {
		try {
			AOADLMetamodel = (ASMEMFModel) modelLoader.loadModel("AOADL", modelLoader.getMOF(), aoadl_ModelResource);
			UMLMetamodel = (ASMEMFModel) modelLoader.loadModel("UML", modelLoader.getMOF(), uml_ModelResource);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
 
		models.put("AOADL", AOADLMetamodel);
		models.put("UML", UMLMetamodel);
		
	}
	
	private void initThemeUMLMetamodels(Map<String, Object> models, Map<String, Object> inLibs) {
		try {
			AOADLMetamodel = (ASMEMFModel) modelLoader.loadModel("AOADL", modelLoader.getMOF(), aoadl_ModelResource);
			UMLMetamodel = (ASMEMFModel) modelLoader.loadModel("UML", modelLoader.getMOF(), uml_ModelResource);
			
			AOADL_PROFILE = (ASMEMFModel) modelLoader.loadModel("AOADL_PROFILE", UMLMetamodel, strAOADL_PROFILE);
			THEMEUML_PROFILE = (ASMEMFModel) modelLoader.loadModel("THEMEUML_PROFILE", UMLMetamodel, strTHEMEUML_PROFILE);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
 
		models.put("AOADL", AOADLMetamodel);
		models.put("UML", UMLMetamodel);
		models.put("AOADL_PROFILE", AOADL_PROFILE);
		models.put("THEMEUML_PROFILE", THEMEUML_PROFILE);
		
		inLibs.put("LIBAOADL2ThemeUML", LIBAOADL2ThemeUML);
		inLibs.put("LIBUMLClone", LIBUMLClone);
	}
	
	public void copyProfiletoLocation(String fileName, String location)throws IOException{
				
		File file = new File(location + "/" +fileName);
		
		if (!file.exists()) {
			InputStream from = null;
			FileOutputStream to = null;
			try {
				from = ATLTransformations.class.getResourceAsStream("profiles/"+fileName);
				to = new FileOutputStream(file);
				byte[] buffer = new byte[4096];
				int bytesRead;

				while ((bytesRead = from.read(buffer)) != -1) {
					to.write(buffer, 0, bytesRead); 
				}				
			} finally {
				if (from != null) {
					from.close();
				}
				if (to != null)	{
					to.close();
				}
			}			
		}
	}
	
	public ATLTransformationError transformAOADLtoThemeUML(String source, String target, String prjName, String strFileName){
		try{
			Map<String, Object> models = new HashMap<String, Object>();
			Map<String, Object> inLibs = new HashMap<String, Object>();
			
			// Copy profiles to locations
			copyProfiletoLocation("AOADL.profile.uml", target);
			copyProfiletoLocation("ThemeUML.profile.uml", target);
			
			createThemeUMLResources("platform:/resource"+prjName+"/DESIGN/");
			initThemeUMLMetamodels(models, inLibs);
		
			// get/create models
			ASMEMFModel aoadlInputModel = (ASMEMFModel) modelLoader.loadModel("aoadlM", AOADLMetamodel, "platform:/resource"+prjName+"/ARCH/"+strFileName);
			ASMEMFModel umlOutputModel = (ASMEMFModel) modelLoader.newModel("umlM", "platform:/resource"+prjName+"/DESIGN/"+strFileName.substring(0, strFileName.indexOf(".")+1)+"uml", UMLMetamodel);
			// load models
			models.put("aoadlM", aoadlInputModel);
			models.put("umlM", umlOutputModel);
			
			// Set options
			Map<String, String> options = new HashMap<String, String>();
			options.put("step", "false");
			options.put("showSummary", "false");
			options.put("profile", "false");
			options.put("continueAfterError", "false");
			options.put("disableAttributeHelperCache", "false");
			options.put("printExecutionTime", "false");
			
			// launch
			AtlLauncher.getDefault().launch(this.aoadl2themeuml_TransfoResource,
					inLibs, models, Collections.EMPTY_MAP,
					Collections.EMPTY_LIST, Collections.EMPTY_MAP);
 
			modelLoader.save(umlOutputModel, "platform:/resource"+prjName+"/DESIGN/"+strFileName.substring(0, strFileName.indexOf(".")+1)+"uml");
			
			//dispose(models);
			
		} catch (Exception e) {
			e.printStackTrace();
		}

		return ATLTransformationError.NO_ERROR;
	}
	
	
	public ATLTransformationError transformPatterntoAOADL(String source, String target, String prjName, String strFileName){
		try{
			Map<String, Object> models = new HashMap<String, Object>();
			
			createAOADLResources();
			initAOADLMetamodels(models);
			
			// get/create models
			ASMEMFModel umlInputModel = (ASMEMFModel) modelLoader.loadModel("umlCM", UMLMetamodel, "platform:/resource"+prjName+"/CM/"+strFileName);
			ASMEMFModel aoadlOutputModel = (ASMEMFModel) modelLoader.newModel("aoadlM", "platform:/resource"+prjName+"/ARCH/"+strFileName.substring(0, strFileName.indexOf(".")+1)+"aoadl", AOADLMetamodel);
			// load models
			models.put("umlCM", umlInputModel);
			models.put("aoadlM", aoadlOutputModel);
			
			// Set options
			Map<String, String> options = new HashMap<String, String>();
			options.put("step", "false");
			options.put("showSummary", "false");
			options.put("profile", "false");
			options.put("continueAfterError", "false");
			options.put("disableAttributeHelperCache", "false");
			options.put("printExecutionTime", "false");
			
			// launch
			AtlLauncher.getDefault().launch(this.uml2aoadl_TransfoResource,
					Collections.EMPTY_MAP, models, Collections.EMPTY_MAP,
					Collections.EMPTY_LIST, Collections.EMPTY_MAP);
			
			modelLoader.save(aoadlOutputModel, "platform:/resource"+prjName+"/ARCH/"+strFileName.substring(0, strFileName.indexOf(".")+1)+"aoadl");
			
			//dispose(models);
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return ATLTransformationError.NO_ERROR;
	}
}
